(*----------------------------------------------------------------------------*
 *  Direct3D sample from DirectX 9.0 SDK Summer 2003                          *
 *  Delphi adaptation by Alexey Barkovoy (e-mail: clootie@ixbt.com)           *
 *                                                                            *
 *  Delphi versions 5-7 are supported                                         *
 *                                                                            *
 *  Modified: 30-Dec-2003                                                     *
 *                                                                            *
 *  Latest version can be downloaded from:                                    *
 *     http://clootie.narod.ru/delphi                                         *
 *----------------------------------------------------------------------------*)
//-----------------------------------------------------------------------------
// File: Fur.cpp
//
// Desc: Example code showing how to do real-time fur rendering.
//
// GEOMETRY
// An original copy of the geometry is maintained in vertex stream 0.
// A copy of the original vertex data is then appended to the same vertex buffer.
// This is because the fins will contain at most two vertices for every base
// vertex (The original vertex, plus its extruded position for the fin extent).
// Likewise, the index buffer contains one copy of the original indices.  These
// indices are used for the base pass as well as the shells.  Appended onto the
// end of these indices are the index data for the fins pass.
// Shell and fin texturing data are generated in a second stream (stream 1).
// VB(0):  VBChunk0 (original vertices) + VBChunk1 (second copy of original vertices)
// IB:     IBChunk0 (original indices) + IBChunk1 (fin indices)
// VB(1a): TexCoordA Shell texture coordinates
// VB(1b): TexCoordB Fin texture coordinates
// VBChunk0 and IBChunk0 comprise the original mesh.
// VBChunk1, IBChunk1, TexCoordA, and TexCoordB are generated in a pre-processing
// stage.
// BASE PASS DESCRIPTION
// Indices:  IBChunk0
// Stream 0: VBChunk0
// Stream 1: Not used
// Shader:   Normal FVF or shader.
// This draws the base object as it otherwise would normally appear.
// SHELL PASS DESCRIPTION
// Indices:  IBChunk0
// Stream 0: VBChunk0
// Stream 1: TexCoordA(Stream 1)
// Shader:   Displaces the shells outward around the mesh.
// FIN PASS DESCRIPTION
// Stream 0: VBChunk0 + VBChunk1
// Stream 1: TexCoordB
// Indices:  IBChunk1
// Shader:   The fin pass vertex shader extrudes the vertices using
// the Stream1 texture v coordinate as an extrusion factor.  The vertex in VBChunk0
// contains an equivalent in VBChunk1, however one will be extruded (by the texture
// coordinates) and the other will not.
//
// Copyright (c) Microsoft Corporation. All rights reserved.
//-----------------------------------------------------------------------------

unit FurUnit;

{$I DirectX.inc}

interface

uses
  Windows, Messages,
  Direct3D9, D3DX9,
  D3DApp, D3DFont, DXUtil, D3DUtil,
  Resource;

{.$DEFINE DEBUG_VS}   // Uncomment this line to debug vertex shaders
{.$DEFINE DEBUG_PS}   // Uncomment this line to debug pixel shaders

type
  // structure contains basic mesh vertex
  PMyVertex = ^TMyVertex;
  TMyVertex = packed record
    x,y,z: Single;
    x0, y0, z0: Single;
    diffuse: DWORD;
  end;

const
  TMyVertex_FVF = DWORD(D3DFVF_XYZ or D3DFVF_NORMAL or D3DFVF_DIFFUSE);

type
  // both the shells and fins add a stream of texture coordinates to augment the basic mesh
  PMyTexCoord = ^TMyTexCoord;
  TMyTexCoord = packed record
    u, v: Single;
  end;



  //-----------------------------------------------------------------------------
  // Name: class CFuzzyMesh
  // Desc: Geometry Class. The base class for a shells & fins mesh.
  //-----------------------------------------------------------------------------
  CFuzzyMesh = class
  public
    procedure Init(pVertices: PMyVertex; pIndices: PWord; pShellTexcoords: PMyTexCoord;
      pFinTexcoords: PMyTexCoord); virtual; abstract;
    function GetNumShellVertices: DWORD; virtual; abstract;
    function GetNumFinVertices: DWORD; virtual; abstract;
    function GetNumShellPrimitives: DWORD; virtual; abstract;
    function GetNumFinPrimitives: DWORD; virtual; abstract;
    class function CreateCube(nWidthSlices, nHeightSlices: LongWord;
      dwColor: DWORD; fTextureScale: Single): CFuzzyMesh;
  end;


  PIDirect3DTexture9Array = ^IDirect3DTexture9Array;
  IDirect3DTexture9Array = array [0..MaxInt div SizeOf(IDirect3DTexture9)-1] of IDirect3DTexture9;



  //-----------------------------------------------------------------------------
  // Name: class CMyD3DApplication
  // Desc: Application class. The base class (CD3DApplication) provides the
  //       generic functionality needed in all Direct3D samples. CMyD3DApplication
  //       adds functionality specific to this sample program.
  //-----------------------------------------------------------------------------
  CMyD3DApplication = class(CD3DApplication)
  protected

    m_pFont:                 CD3DFont;
    m_bBasePass:             Boolean;
    m_bShellPass:            Boolean;
    m_bFinPass:              Boolean;
    m_nShells:               LongWord;
    m_pFuzzyMesh:            CFuzzyMesh;
    m_pDeclaration:          IDirect3DVertexDeclaration9;
    m_pVB:                   IDirect3DVertexBuffer9;
    m_pShellTexVB:           IDirect3DVertexBuffer9;
    m_pFinTexVB:             IDirect3DVertexBuffer9;
    m_pIB:                   IDirect3DIndexBuffer9;

    m_pShellTextures:        PIDirect3DTexture9Array;
    m_pFinTexture:           IDirect3DTexture9;
    m_pEffect:               ID3DXEffect;
    m_dwNumShellVertices:    LongWord;
    m_dwNumShellPrimitives:  LongWord;
    m_dwNumFinVertices:      LongWord;
    m_dwNumFinPrimitives:    LongWord;
    m_hBaseTechnique:        TD3DXHandle;
    m_hFinsTechnique:        TD3DXHandle;
    m_hShellTechnique:       TD3DXHandle;
    m_hShellTexture:         TD3DXHandle;
    m_hShellScalingFactor:   TD3DXHandle;
    m_hWorldTransform:       TD3DXHandle;

  protected
    function ConfirmDevice(const pCaps: TD3DCaps9; dwBehavior: DWORD;
      adapterFormat: TD3DFormat; backBufferFormat: TD3DFormat): HRESULT; override;
    function OneTimeSceneInit: HRESULT; override;
    function Render: HRESULT; override;
    function FrameMove: HRESULT; override;

    function InitDeviceObjects: HRESULT; override;
    function RestoreDeviceObjects: HRESULT; override;
    function InvalidateDeviceObjects: HRESULT; override;
    function DeleteDeviceObjects: HRESULT; override;
    function FinalCleanup: HRESULT; override;

    procedure SetMenuStates;
  public
    constructor Create;
    
    function MsgProc(hWnd: HWND; uMsg: LongWord; wParam: WPARAM; lParam: LPARAM): LRESULT; override;
  end;

implementation

uses SysUtils;


{ CMyD3DApplication }

//-----------------------------------------------------------------------------
// Name: CMyD3DApplication()
// Desc: Application constructor. Sets attributes for the app.
//-----------------------------------------------------------------------------
constructor CMyD3DApplication.Create;
begin
  inherited Create;
  m_strWindowTitle    := 'Fur: Fur rendering using shells and fins';
  m_d3dEnumeration.AppUsesDepthBuffer := True;

  // intrinsic
  m_pFont                := CD3DFont.Create('Arial', 12, D3DFONT_BOLD); 
  m_dwNumShellVertices   := 0;
  m_dwNumShellPrimitives := 0;
  m_dwNumFinVertices     := 0;
  m_dwNumFinPrimitives   := 0;
  m_pFuzzyMesh           := nil;

  // D3D Resources
  m_pVB            := nil;
  m_pIB            := nil;
  m_pDeclaration   := nil;
  m_pEffect        := nil;
  m_pShellTexVB    := nil;
  m_pFinTexVB      := nil;
  m_pShellTextures := nil;
  m_pFinTexture    := nil;

  // render control
  m_bBasePass  := True;
  m_bShellPass := True;
  m_bFinPass   := True;
  m_nShells := 16;
end;

//-----------------------------------------------------------------------------
// Name: ConfirmDevice()
// Desc: Called during device initialization, this code checks the device
//       for some minimum set of capabilities
//-----------------------------------------------------------------------------
function CMyD3DApplication.ConfirmDevice(const pCaps: TD3DCaps9;
  dwBehavior: DWORD; adapterFormat, backBufferFormat: TD3DFormat): HRESULT;
begin
  Result:= E_FAIL;

  // Debugging vertex shaders requires either REF or software vertex processing
  // and debugging pixel shaders requires REF.
  {$IFDEF DEBUG_VS}
    if (pCaps.DeviceType <> D3DDEVTYPE_REF) and
       ((dwBehavior and D3DCREATE_SOFTWARE_VERTEXPROCESSING) = 0)
    then Exit;
  {$ENDIF}
  {$IFDEF DEBUG_PS}
    if (pCaps.DeviceType <> D3DDEVTYPE_REF) 
    then Exit;
  {$ENDIF}

  if (dwBehavior and D3DCREATE_HARDWARE_VERTEXPROCESSING <> 0) or
     (dwBehavior and D3DCREATE_MIXED_VERTEXPROCESSING <> 0) then
  begin
    if (pCaps.VertexShaderVersion < D3DVS_VERSION(1,1)) then Exit;
  end;

  Result:= S_OK;
end;

//-----------------------------------------------------------------------------
// Name: OneTimeSceneInit()
// Desc: Called during initial app startup, this function performs all the
//       permanent initialization.
//-----------------------------------------------------------------------------
function CMyD3DApplication.OneTimeSceneInit: HRESULT;
begin
  m_pFuzzyMesh := CFuzzyMesh.CreateCube(8, 8, D3DCOLOR_XRGB($60, $a0, $40), 0.125);
  if (nil = m_pFuzzyMesh) then
  begin
    Result:= D3DAPPERR_MEDIANOTFOUND;
    Exit;
  end;

  Result:= S_OK;
end;

//-----------------------------------------------------------------------------
// Name: InitDeviceObjects()
// Desc: Initialize scene objects.
//-----------------------------------------------------------------------------
var
  // Declaration - used for shell and fin passes
  declShell: array[0..4] of TD3DVertexElement9 =
  (
    (Stream: 0; Offset: 0; _Type: D3DDECLTYPE_FLOAT3; Method: D3DDECLMETHOD_DEFAULT; Usage: D3DDECLUSAGE_POSITION; UsageIndex: 0),
    (Stream: 0; Offset: 12; _Type: D3DDECLTYPE_FLOAT3; Method: D3DDECLMETHOD_DEFAULT; Usage: D3DDECLUSAGE_NORMAL; UsageIndex: 0),
    (Stream: 0; Offset: 24; _Type: D3DDECLTYPE_D3DCOLOR; Method: D3DDECLMETHOD_DEFAULT; Usage: D3DDECLUSAGE_COLOR; UsageIndex: 0),
    (Stream: 1; Offset: 0; _Type: D3DDECLTYPE_FLOAT2; Method: D3DDECLMETHOD_DEFAULT; Usage: D3DDECLUSAGE_TEXCOORD; UsageIndex: 0),
    ({D3DDECL_END})
  );

function CMyD3DApplication.InitDeviceObjects: HRESULT;
var
  strPath: array[0..MAX_PATH-1] of Char;
  dwShaderFlags: DWORD;
begin
  m_pFont.InitDeviceObjects(m_pd3dDevice);

  // Declaration - used for shell and fin passes
  declShell[4]:= D3DDECL_END;

  Result:= m_pd3dDevice.CreateVertexDeclaration(@declShell, m_pDeclaration);
  if FAILED(Result) then Exit;

  // Define DEBUG_VS and/or DEBUG_PS to debug vertex and/or pixel shaders with the shader debugger.
  // Debugging vertex shaders requires either REF or software vertex processing, and debugging
  // pixel shaders requires REF.  The D3DXSHADER_FORCE_*_SOFTWARE_NOOPT flag improves the debug
  // experience in the shader debugger.  It enables source level debugging, prevents instruction
  // reordering, prevents dead code elimination, and forces the compiler to compile against the next
  // higher available software target, which ensures that the unoptimized shaders do not exceed
  // the shader model limitations.  Setting these flags will cause slower rendering since the shaders
  // will be unoptimized and forced into software.  See the DirectX documentation for more information
  // about using the shader debugger.
  dwShaderFlags := 0;
  {$IFDEF DEBUG_VS}
    dwShaderFlags := dwShaderFlags or D3DXSHADER_FORCE_VS_SOFTWARE_NOOPT;
  {$ENDIF}
  {$IFDEF DEBUG_PS}
    dwShaderFlags := dwShaderFlags or D3DXSHADER_FORCE_PS_SOFTWARE_NOOPT;
  {$ENDIF}

  // Load the effect file
  Result:= DXUtil_FindMediaFileCb(strPath, SizeOf(strPath), 'Fur.fx');
  if FAILED(Result) then Exit;
  Result:= D3DXCreateEffectFromFile(m_pd3dDevice, strPath, nil, nil, dwShaderFlags, nil, m_pEffect, nil);
  if FAILED(Result) then Exit;

  // make sure the render pass options are set in the menu
  SetMenuStates;

  Result:= S_OK;
end;

//-----------------------------------------------------------------------------
// Name: RestoreDeviceObjects()
// Desc: Initialize scene objects.
//-----------------------------------------------------------------------------
function CMyD3DApplication.RestoreDeviceObjects: HRESULT;
var
  pBaseVertex: PMyVertex;
  pShellTexVerts: PMyTexCoord;
  pFinTexVerts: PMyTexCoord;
  pBaseIndex: PWord;
  strPath: array[0..MAX_PATH-1] of Char;
  szFileName: array[0..MAX_PATH-1] of Char;
  i: LongWord;
  dwBufferSize: DWORD;
  view, projection: TD3DXMatrixA16;
  Eye: TD3DXVector3;
  At: TD3DXVector3;
  Up: TD3DXVector3;
  numPasses: LongWord;
  iPass: LongWord;
begin
  pBaseVertex := nil;
  pBaseIndex := nil;

  m_pFont.RestoreDeviceObjects;

  // create the shell textures
  try
    GetMem(m_pShellTextures, m_nShells*SizeOf(IDirect3DTexture9));
  except
    Result:= E_FAIL;
    Exit;
  end;

  // initialize the array of shell textures to NULL
  FillChar(m_pShellTextures^, m_nShells*SizeOf(IDirect3DTexture9), 0);

  // the filename buffer
  szFileName[MAX_PATH-1]:= #0;

  // load each shell level
  for i := 0 to m_nShells - 1 do
  begin
    StrLFmt(szFileName, MAX_PATH-1, 'fur_%.2d.dds', [i]);

    Result:= D3DAPPERR_MEDIANOTFOUND;

    // Get the path to the texture
    if FAILED(DXUtil_FindMediaFileCb(strPath, sizeof(strPath), szFileName)) then Exit;
    // Create the texture using D3DX
    if FAILED(D3DXCreateTextureFromFileEx(m_pd3dDevice, strPath,
                D3DX_DEFAULT, D3DX_DEFAULT, D3DX_DEFAULT, 0, D3DFMT_A8R8G8B8,
                D3DPOOL_MANAGED, D3DX_FILTER_TRIANGLE or D3DX_FILTER_MIRROR,
                D3DX_FILTER_TRIANGLE or D3DX_FILTER_MIRROR, 0, nil, nil, m_pShellTextures[i]))
    then Exit;
  end;

  // create the fin texture
  if FAILED(D3DUtil_CreateTexture(m_pd3dDevice, 'fur_fin_00.dds',
                                  m_pFinTexture, D3DFMT_A8R8G8B8)) then
  begin
    Result:= D3DAPPERR_MEDIANOTFOUND;
    Exit;
  end;

  // proceed to load mesh data into the D3D Resources
  if (nil = m_pFuzzyMesh) then
  begin
    Result:= D3DAPPERR_MEDIANOTFOUND;
    Exit;
  end;

  // save the number of vertices/primitives in the geometry
  m_dwNumShellVertices := m_pFuzzyMesh.GetNumShellVertices;
  m_dwNumShellPrimitives := m_pFuzzyMesh.GetNumShellPrimitives;
  m_dwNumFinVertices := m_pFuzzyMesh.GetNumFinVertices;
  m_dwNumFinPrimitives := m_pFuzzyMesh.GetNumFinPrimitives;

  dwBufferSize := m_dwNumShellPrimitives * 3 * SizeOf(Word) + m_dwNumFinPrimitives * 3 * SizeOf(Word);
  Result := m_pd3dDevice.CreateIndexBuffer(dwBufferSize, D3DUSAGE_WRITEONLY, D3DFMT_INDEX16, D3DPOOL_DEFAULT, m_pIB, nil);
  if FAILED(Result) then Exit;

  dwBufferSize := m_dwNumFinVertices * SizeOf(TMyVertex);
  Result := m_pd3dDevice.CreateVertexBuffer(dwBufferSize, D3DUSAGE_WRITEONLY, 0, D3DPOOL_DEFAULT, m_pVB, nil);
  if FAILED(Result) then Exit;

  dwBufferSize := m_dwNumShellVertices * SizeOf(TMyTexCoord);
  Result := m_pd3dDevice.CreateVertexBuffer(dwBufferSize, D3DUSAGE_WRITEONLY, 0, D3DPOOL_DEFAULT, m_pShellTexVB, nil);
  if FAILED(Result) then Exit;

  dwBufferSize := m_dwNumFinVertices * SizeOf(TMyTexCoord);
  Result := m_pd3dDevice.CreateVertexBuffer(dwBufferSize, D3DUSAGE_WRITEONLY, 0, D3DPOOL_DEFAULT, m_pFinTexVB, nil);
  if FAILED(Result) then Exit;

  Result := m_pVB.Lock(0, 0, Pointer(pBaseVertex), 0);
  if FAILED(Result) then Exit;

  Result := m_pShellTexVB.Lock(0, 0, Pointer(pShellTexVerts), 0);
  if FAILED(Result) then Exit;

  Result := m_pFinTexVB.Lock(0, 0, Pointer(pFinTexVerts), 0);
  if FAILED(Result) then Exit;

  Result := m_pIB.Lock(0, 0, Pointer(pBaseIndex), 0);
  if FAILED(Result) then Exit;

  // Load the mesh data into the buffers
  m_pFuzzyMesh.Init(pBaseVertex, pBaseIndex, pShellTexVerts, pFinTexVerts);

  // unlock the buffers and proceed
  m_pIB.Unlock;
  m_pFinTexVB.Unlock;
  m_pShellTexVB.Unlock;
  m_pVB.Unlock;
    
  // save frequently used handles
  m_hBaseTechnique      := m_pEffect.GetTechniqueByName('Base');
  m_hFinsTechnique      := m_pEffect.GetTechniqueByName('Fins');
  m_hShellTechnique     := m_pEffect.GetTechniqueByName('Shells');
  m_hShellTexture       := m_pEffect.GetParameterByName(nil, 'ShellTex');
  m_hShellScalingFactor := m_pEffect.GetParameterByName(nil, 'ShellScalingFactor');
  m_hWorldTransform     := m_pEffect.GetParameterBySemantic(nil, 'WORLD');

  Eye := D3DXVector3(0.0, 0.0, -5.1);
  At  := D3DXVector3(0.0, 0.0, 0.0);
  Up  := D3DXVector3(0.0, 1.0, 0.0);
    
  // Eye position
  m_pEffect.SetValue(m_pEffect.GetParameterByName(nil, 'Eye'), @Eye, SizeOf(TD3DXVector3));

  // establish view matrix    
  D3DXMatrixLookAtLH(view, Eye, At, Up);
  m_pEffect.SetMatrix(m_pEffect.GetParameterBySemantic(nil, 'VIEW'), view);
    
  // establish projection matrix
  D3DXMatrixPerspectiveFovLH(projection, D3DX_PI*0.25,m_dwCreationWidth / m_dwCreationHeight, 0.1, 100.0);
  m_pEffect.SetMatrix(m_pEffect.GetParameterBySemantic(nil, 'PROJECTION'), projection);

  // Fin texture
  m_pEffect.SetTexture(m_pEffect.GetParameterByName(nil, 'FinTex'), m_pFinTexture);

  // Set default states for the application
  m_pEffect.SetTechnique(m_pEffect.GetTechniqueByName('Init'));
  m_pEffect._Begin(@numPasses, D3DXFX_DONOTSAVESTATE);
  for iPass := 0 to numPasses - 1 do
  begin
    m_pEffect.Pass(iPass);
  end;
  m_pEffect._End;

  if (m_pEffect <> nil) then
    m_pEffect.OnResetDevice;

  Result:= S_OK;
end;

//-----------------------------------------------------------------------------
// Name: InvalidateDeviceObjects()
// Desc:
//-----------------------------------------------------------------------------
function CMyD3DApplication.InvalidateDeviceObjects: HRESULT;
var
  i: Integer;
begin
  m_pFont.InvalidateDeviceObjects;
  SAFE_RELEASE(m_pIB);
  SAFE_RELEASE(m_pVB);
  SAFE_RELEASE(m_pShellTexVB);
  SAFE_RELEASE(m_pFinTexVB);
  SAFE_RELEASE(m_pFinTexture);
  if (m_pShellTextures <> nil) then
  begin
    for i := 0 to m_nShells - 1 do
    begin
      SAFE_RELEASE(m_pShellTextures[i]);
    end;
    FreeMem(m_pShellTextures); // SAFE_DELETE_ARRAY( m_pShellTextures );
  end;

  if (m_pEffect <> nil) then
    m_pEffect.OnLostDevice;

  Result:= S_OK;
end;

//-----------------------------------------------------------------------------
// Name: DeleteDeviceObjects()
// Desc: Called when the app is exiting, or the device is being changed,
//       this function deletes any device dependent objects.
//-----------------------------------------------------------------------------
function CMyD3DApplication.DeleteDeviceObjects: HRESULT;
begin
  m_pFont.DeleteDeviceObjects; 
  SAFE_RELEASE(m_pDeclaration);
  SAFE_RELEASE(m_pEffect);

  Result:= S_OK;
end;

//-----------------------------------------------------------------------------
// Name: FinalCleanup()
// Desc: Called before the app exits, this function gives the app the chance
//       to cleanup after itself.
//-----------------------------------------------------------------------------
function CMyD3DApplication.FinalCleanup: HRESULT;
begin
  SAFE_DELETE(m_pFuzzyMesh);
//  SAFE_DELETE(m_pFont);
  Result:= S_OK;
end;

//-----------------------------------------------------------------------------
// Name: FrameMove()
// Desc: Called once per frame, the call is the entry point for animating
//       the scene.
//-----------------------------------------------------------------------------
function CMyD3DApplication.FrameMove: HRESULT;
var
  mat, world: TD3DXMatrixA16;
begin
  // establish world matrix
  D3DXMatrixIdentity(world);
  D3DXMatrixMultiply(world, world, D3DXMatrixRotationY(mat, m_fTime * D3DX_PI * 0.2)^);
  D3DXMatrixMultiply(world, world, D3DXMatrixRotationX(mat, m_fTime * D3DX_PI * 0.2)^);

  m_pEffect.SetMatrix(m_hWorldTransform, world);

  Result:= S_OK;
end;

//-----------------------------------------------------------------------------
// Name: Render()
// Desc: Called once per frame, the call is the entry point for 3d
//       rendering. This function sets up render states, clears the
//       viewport, and renders the scene.
//-----------------------------------------------------------------------------
function CMyD3DApplication.Render: HRESULT;
var
  numPasses: LongWord;
  iPass: LongWord;
  iShell: LongWord;
begin
  // clear
  m_pd3dDevice.Clear(0, nil, D3DCLEAR_TARGET or D3DCLEAR_ZBUFFER,
    $000000ff, 1.0, 0);

  if SUCCEEDED(m_pd3dDevice.BeginScene) then
  begin
    // The main MYVERTEX vertex buffer is used for all 3 passes
    m_pd3dDevice.SetStreamSource(0, m_pVB, 0, SizeOf(TMyVertex));

    // render the base object
    if m_bBasePass then
    begin
      m_pd3dDevice.SetStreamSource(1, nil, 0, 0);
      m_pd3dDevice.SetIndices(m_pIB);

      m_pEffect.SetTechnique(m_hBaseTechnique);
      m_pEffect._Begin(@numPasses, 0);
      for iPass := 0 to numPasses - 1 do
      begin
        m_pEffect.Pass(iPass);
        m_pd3dDevice.DrawIndexedPrimitive(D3DPT_TRIANGLELIST, 0, 0, m_dwNumShellVertices, 0, m_dwNumShellPrimitives);
      end;
      m_pEffect._End;
    end;

    // render fins
    if m_bFinPass then
    begin
      m_pd3dDevice.SetVertexDeclaration(m_pDeclaration);
      m_pd3dDevice.SetStreamSource(1, m_pFinTexVB, 0, SizeOf(TMyTexCoord));
      m_pd3dDevice.SetIndices(m_pIB);

      m_pEffect.SetTechnique(m_hFinsTechnique);
      m_pEffect._Begin(@numPasses, 0);
      for iPass := 0 to numPasses - 1 do
      begin
        m_pEffect.Pass(iPass);
        m_pd3dDevice.DrawIndexedPrimitive(D3DPT_TRIANGLELIST, 0, 0, m_dwNumFinVertices, m_dwNumShellPrimitives*3, m_dwNumFinPrimitives);
      end;
      m_pEffect._End;
    end;

    // render shells
    if m_bShellPass then
    begin
      m_pd3dDevice.SetVertexDeclaration(m_pDeclaration);
      m_pd3dDevice.SetStreamSource(1, m_pShellTexVB, 0, SizeOf(TMyTexCoord));
      m_pd3dDevice.SetIndices(m_pIB);

      m_pEffect.SetTechnique(m_hShellTechnique);
      m_pEffect._Begin(@numPasses, 0);
      for iPass := 0 to numPasses - 1 do
      begin
        m_pEffect.Pass(iPass);
        for iShell := 0 to m_nShells - 1 do
        begin
          // set the texture for this layer
          m_pEffect.SetTexture(m_hShellTexture, m_pShellTextures[iShell]);
          m_pEffect.SetFloat(m_hShellScalingFactor, (iShell + 1) / m_nShells);

          m_pd3dDevice.DrawIndexedPrimitive(D3DPT_TRIANGLELIST, 0, 0, m_dwNumShellVertices, 0, m_dwNumShellPrimitives);
        end;
      end;
      m_pEffect._End;
    end;

    // Dump out the FPS
    m_pFont.DrawText(2,  0, D3DCOLOR_ARGB(255,255,255,0), m_strFrameStats);
    m_pFont.DrawText(2, 20, D3DCOLOR_ARGB(255,255,255,0), m_strDeviceStats);
    m_pd3dDevice.EndScene;
  end;

  Result:= S_OK;
end;

//-----------------------------------------------------------------------------
// Name: MsgProc()
// Desc: Message proc function to handle key and menu input
//-----------------------------------------------------------------------------
function CMyD3DApplication.MsgProc(hWnd: HWND; uMsg: LongWord;
  wParam: WPARAM; lParam: LPARAM): LRESULT;
begin
  // Handle key presses
  case uMsg of
    WM_COMMAND:
    begin
      case LOWORD(wParam) of
        IDM_BASE:   m_bBasePass  := not m_bBasePass;
        IDM_SHELLS: m_bShellPass := not m_bShellPass;
        IDM_FINS:   m_bFinPass   := not m_bFinPass;
      end; 

      // Update the menus, in case any state changes occurred
      SetMenuStates;
    end;
  end;

  // Pass remaining messages to default handler
  Result:= inherited MsgProc(hWnd, uMsg, wParam, lParam);
end;


//-----------------------------------------------------------------------------
// Name: SetMenuStates()
// Desc:
//-----------------------------------------------------------------------------
procedure CMyD3DApplication.SetMenuStates;
var
  hMenu: Windows.HMENU;
const
  IsChecked: array[False..True] of Integer = (MF_UNCHECKED, MF_CHECKED);
begin
  hMenu := GetMenu(m_hWnd);

  CheckMenuItem(hMenu, IDM_BASE, IsChecked[m_bBasePass]);
  CheckMenuItem(hMenu, IDM_SHELLS, IsChecked[m_bShellPass]);
  CheckMenuItem(hMenu, IDM_FINS, IsChecked[m_bFinPass]);
end;


//-----------------------------------------------------------------------------
// Q: What would it take to render an arbitrary mesh?
// A: The problem is in generating texture coordinates.
//    A patching algorithm would need to be implemented to apply texture patches
//    to the mesh in a locally flattened manner
//    Fins would then be grown from the mesh along the seams of such patches.
//-----------------------------------------------------------------------------

type
  //-----------------------------------------------------------------------------
  // Name: class CFuzzyCubeMesh
  // Desc: Implements a proceedural fuzzy cube mesh
  //-----------------------------------------------------------------------------
  CFuzzyCubeMesh = class(CFuzzyMesh)
  protected
    m_nWidthSlices: LongWord;
    m_nHeightSlices: LongWord;
    m_dwColor: DWORD;
    m_fTextureScale: Single;
    m_dwNumShellVertices: LongWord;
    m_dwNumShellPrimitives: LongWord;
    m_dwNumFinVertices: LongWord;
    m_dwNumFinPrimitives: LongWord;
  public
    constructor Create(WidthSlices, nHeightSlices: LongWord; dwColor: DWORD; fTextureScale: Single);
    procedure Init(pVertices: PMyVertex; pIndices: PWord; pShellTexcoords, pFinTexcoords: PMyTexCoord); override;
    procedure TesselateQuad(const normal, Left: TD3DVector; pVertices: PMyVertex;
        pIndices: PWord; dwIndexOffset: Word);
    procedure FinQuad(wExtOffset{wExtrusionOffset}: Word; pShellTexCoords, pFinTexCoords: PMyTexCoord; pIndices: PWord);
    procedure QuadVertex(const normal, Left, Up: TD3DVector; x, y: LongWord;
        x_max, y_max: LongWord; pVertex: PMyVertex);
    function GetNumShellVertices: DWORD; override;   { return m_dwNumShellVertices; }
    function GetNumFinVertices: DWORD; override;     { return m_dwNumFinVertices; }
    function GetNumShellPrimitives: DWORD; override; { return m_dwNumShellPrimitives; }
    function GetNumFinPrimitives: DWORD; override;   { return m_dwNumFinPrimitives; }
  end;

{ CFuzzyCubeMesh }

//-----------------------------------------------------------------------------
// Name: CFuzzyCubeMesh
// Desc: Constructor - calculates mesh expanses for later use
//-----------------------------------------------------------------------------
constructor CFuzzyCubeMesh.Create(WidthSlices, nHeightSlices: LongWord;
  dwColor: DWORD; fTextureScale: Single);
begin
  m_nWidthSlices:= WidthSlices;
  m_nHeightSlices:= nHeightSlices;
  m_dwColor:= dwColor;
  m_fTextureScale:= fTextureScale;

  // save the number of vertices in the base/shells mesh
  m_dwNumShellVertices := 6 * (m_nWidthSlices+1) * (m_nHeightSlices+1);
  m_dwNumShellPrimitives := 6 * 2 * m_nWidthSlices * m_nHeightSlices;

  // for every shell vert there are two face vert
  m_dwNumFinVertices := m_dwNumShellVertices shl 1;
  m_dwNumFinPrimitives := 6 * 2 * ((m_nWidthSlices+1) * (m_nHeightSlices)  + (m_nWidthSlices) * (m_nHeightSlices+1));
end;

//-----------------------------------------------------------------------------
// Name: Init()
// Desc: Returns the vertices, indices, and texcoords
//-----------------------------------------------------------------------------
const
  face: array[0..11] of TD3DVector = (
   (x: 1.0; y: 0.0; z: 0.0),   (x:  0.0; y:  1.0; z:  0.0),
   (x: 0.0; y: 1.0; z: 0.0),   (x: -1.0; y:  0.0; z:  0.0),
   (x: 0.0; y: 0.0; z: 1.0),   (x:  1.0; y:  0.0; z:  0.0),

   (x: -1.0; y: 0.0; z: 0.0),  (x: 0.0; y: -1.0; z:  0.0),
   (x: 0.0; y: -1.0; z: 0.0),  (x: -1.0; y:  0.0; z:  0.0),
   (x: 0.0; y: 0.0; z: -1.0),  (x: -1.0; y:  0.0; z:  0.0)
  );

procedure CFuzzyCubeMesh.Init(pVertices: PMyVertex; pIndices: PWord;
  pShellTexcoords, pFinTexcoords: PMyTexCoord);
var
  wIndexOffset: Word;
  pBaseVertexStart: PMyVertex;
  dwPrimsPerFace: DWORD;
  dwVertsPerFace: DWORD;
  faces: Cardinal;
begin
  wIndexOffset := 0;
  pBaseVertexStart := pVertices;
  dwPrimsPerFace := m_dwNumShellPrimitives div 6;
  dwVertsPerFace := m_dwNumShellVertices div 6;

  for faces:= 0 to 5 do
  begin
    TesselateQuad(face[2*faces], face[2*faces+1], pVertices, pIndices, wIndexOffset);
    Inc(pVertices,    dwVertsPerFace);
    Inc(wIndexOffset, dwVertsPerFace);
    Inc(pIndices,     dwPrimsPerFace * 3);
  end;
  // to extrude the vertices, we need an identical copy of the tesselated vertices
  // appended to the end of the tesselation
  CopyMemory(pVertices, pBaseVertexStart, m_dwNumShellVertices*SizeOf(TMyVertex));

  // generate the fin texture coordinates and indices
  FinQuad(m_dwNumShellVertices, pShellTexcoords, pFinTexcoords, pIndices);
end;

//-----------------------------------------------------------------------------
// Name: TesselateQuad()
// Desc: Tesselates one Quad/Cube Face
//-----------------------------------------------------------------------------
procedure CFuzzyCubeMesh.TesselateQuad(const normal, Left: TD3DVector;
  pVertices: PMyVertex; pIndices: PWord; dwIndexOffset: Word);
var
  Up: TD3DXVector3;
  height: LongWord;
  width: LongWord;
begin
  // given the general surface normal, and a "left" width-traversal vector, calculate an "up" height-traversal direction
  D3DXVec3Cross(Up, normal, Left);

  // loop through each vertex, left to right and top to bottom
  for height := 0 to m_nHeightSlices do
  begin
    for width := 0 to m_nWidthSlices do
    begin
      // generate the vertex position, normal, and diffuse color
      QuadVertex(normal, Left, Up, width, height, m_nWidthSlices, m_nHeightSlices, pVertices);
      Inc(pVertices)
    end;
  end;

  // create the index buffer members, quads starting from "upper left"
  for height := 0 to m_nHeightSlices - 1 do
  begin
    for width := 0 to m_nWidthSlices - 1 do
    begin
      // first triangle of quad
      pIndices^:= dwIndexOffset + (height)*(m_nWidthSlices+1) + width;       Inc(pIndices);  // top left
      pIndices^:= dwIndexOffset + (height)*(m_nWidthSlices+1) + (width+1);   Inc(pIndices);  // top right
      pIndices^:= dwIndexOffset + (height+1)*(m_nWidthSlices+1) + width;     Inc(pIndices);  // bottom left
      // second triangle
      pIndices^:= dwIndexOffset + (height+1)*(m_nWidthSlices+1) + width;     Inc(pIndices);  // bottom left
      pIndices^:= dwIndexOffset + (height)*(m_nWidthSlices+1) + (width+1);   Inc(pIndices);  // top right
      pIndices^:= dwIndexOffset + (height+1)*(m_nWidthSlices+1) + (width+1); Inc(pIndices);  // bottom right
    end;
  end;
end;

//-----------------------------------------------------------------------------
// Name: FinQuad()
// Desc: Generates fins matching TesselateQuad
//-----------------------------------------------------------------------------
procedure CFuzzyCubeMesh.FinQuad(wExtOffset: Word;
  pShellTexCoords, pFinTexCoords: PMyTexCoord; pIndices: PWord);
var
  vertices_per_quad_face: Word;
  face: LongWord;
  height, width: LongWord;
  pFinTexCoordTemp: PMyTexCoord;
  dwIndexOffset: Word;
  base_index: Word;
begin
  vertices_per_quad_face := (m_nHeightSlices+1)*(m_nWidthSlices+1);

  // anchored vertices
  for face := 0 to 5 do
  begin
    for height := 0 to m_nHeightSlices do
    begin
      for width := 0 to m_nWidthSlices do
      begin
        // set the shell tex coords
        pShellTexCoords.u := m_fTextureScale * width;
        pShellTexCoords.v := m_fTextureScale * height;
        Inc(pShellTexCoords);

        // scale the texture larger by a factor of two
        pFinTexCoords.u := m_fTextureScale * (width+height) * 0.5;
        // this is the anchored vertex
        pFinTexCoords.v := 0.0;

        pFinTexCoordTemp:= pFinTexCoords;  Inc(pFinTexCoordTemp, wExtOffset);
        pFinTexCoordTemp.u := m_fTextureScale * (width+height) * 0.5;
        pFinTexCoordTemp.v := 1.0;
        Inc(pFinTexCoords);
      end;
    end;
  end;

  dwIndexOffset := 0;
  // create "horizontal" fins
  for face := 0 to 5 do
  begin
    for height := 0 to m_nHeightSlices do
    begin
      for width := 0 to m_nWidthSlices - 1 do
      begin
        // first triangle of quad
        base_index := dwIndexOffset + (height)*(m_nWidthSlices+1) + width;
        pIndices^ := base_index;                    Inc(pIndices);   // left base
        pIndices^ := base_index + wExtOffset;       Inc(pIndices);   // left extruded
        pIndices^ := base_index + 1;                Inc(pIndices);   // right base
        // second triangle
        pIndices^ := base_index + 1;                Inc(pIndices);   // right base
        pIndices^ := base_index + wExtOffset;       Inc(pIndices);   // left extruded
        pIndices^ := base_index + wExtOffset + 1;   Inc(pIndices);   // right extruded
      end;
    end;
    Inc(dwIndexOffset, vertices_per_quad_face);
  end;

  // create the "vertical" fins
  dwIndexOffset := 0;
  for face := 0 to 5 do
  begin
    for width := 0 to m_nWidthSlices do
    begin
      for height := 0 to m_nHeightSlices - 1 do
      begin
        base_index := dwIndexOffset + (height)*(m_nWidthSlices+1) + width;
        // first triangle of quad
        pIndices^ := base_index;                                     Inc(pIndices);  // top base
        pIndices^ := base_index + wExtOffset;                        Inc(pIndices);  // top extruded
        pIndices^ := base_index + (m_nWidthSlices + 1);              Inc(pIndices);  // bottom base
        // second triangle
        pIndices^ := base_index + (m_nWidthSlices + 1);              Inc(pIndices);  // bottom base
        pIndices^ := base_index + wExtOffset;                        Inc(pIndices);  // top extruded
        pIndices^ := base_index + (m_nWidthSlices + 1) + wExtOffset; Inc(pIndices);  // bottom extruded
      end;
    end;
    Inc(dwIndexOffset, vertices_per_quad_face);
  end;
end;


//-----------------------------------------------------------------------------
// Name: QuadVertex()
// Desc: Generates vertex (minus tex coords) for a 2-d quad, scaled -1 to 1
//
//-----------------------------------------------------------------------------
procedure CFuzzyCubeMesh.QuadVertex(const normal, Left, Up: TD3DVector; x,
  y, x_max, y_max: LongWord; pVertex: PMyVertex);
var
  here, t: TD3DXVector3;
  total_normal: TD3DXVector3;
begin
  // starting 0,0,0 move this vertex outwards by the normal direction
  here := normal;
  // travel all the way to the left of this face grid
  D3DXVec3Add(here, here, Left);
  // travel now to the upper left corner
  D3DXVec3Add(here, here, Up);
  // move back to the right, scaled by the correct position for this vertex
  D3DXVec3Subtract(here, here, D3DXVec3Scale(t, Left, 2*x/x_max)^);
  // move back downwards, scaled by the correct position for this vertex
  D3DXVec3Subtract(here, here, D3DXVec3Scale(t, Up, 2*y/y_max)^);
    
  // we track a "total" normal whereas the edges will have blended normals
  total_normal := normal;
  // special case the edges
  if (0 = y) then
    D3DXVec3Add(total_normal, total_normal, Up)
  else if (y_max = y) then
    D3DXVec3Subtract(total_normal, total_normal, Up);
    
  if (0 = x) then
    D3DXVec3Add(total_normal, total_normal, Left)
  else if (x_max = x) then
    D3DXVec3Subtract(total_normal, total_normal, Left);

  // now normalize the new normal
  D3DXVec3Normalize(total_normal, total_normal);

  // set the vertex position
  pVertex.x := here.x;
  pVertex.y := here.y;
  pVertex.z := here.z;

  // set the vertex normal
  pVertex.x0 := total_normal.x;
  pVertex.y0 := total_normal.y;
  pVertex.z0 := total_normal.z;

  // diffuse color
  pVertex.diffuse := $ff000000 or m_dwColor;
end;

function CFuzzyCubeMesh.GetNumFinPrimitives: DWORD;
begin
  Result:= m_dwNumFinPrimitives;
end;

function CFuzzyCubeMesh.GetNumFinVertices: DWORD;
begin
  Result:= m_dwNumFinVertices;
end;

function CFuzzyCubeMesh.GetNumShellPrimitives: DWORD;
begin
  Result:= m_dwNumShellPrimitives;
end;

function CFuzzyCubeMesh.GetNumShellVertices: DWORD;
begin
  Result:= m_dwNumShellVertices;
end;



{ CFuzzyMesh }

//-----------------------------------------------------------------------------
// Name: CreateCube
// Desc: create a Shells & Fins cube fuzzy mesh
//-----------------------------------------------------------------------------
class function CFuzzyMesh.CreateCube(nWidthSlices, nHeightSlices: LongWord;
  dwColor: DWORD; fTextureScale: Single): CFuzzyMesh;
begin
  Result:= CFuzzyCubeMesh.Create(nWidthSlices, nHeightSlices, dwColor, fTextureScale);
end;

end.

